/**
 * Based on examples in
 *     Head First Object-Oriented Analysis and Design
 *     by Brett D. McLaughlin, Gary Pollice, and David West
 *     O'Reilly, 2007
 *     ISBN 978-0-596-00867-3
 *
 * Ported from Java by
 *     Ronald Mak
 *     Department of Computer Science
 *     San Jose State University
 *     San Jose, CA  95192
 */
#include <vector>
#include <algorithm>
#include "Inventory.h"
#include "Guitar.h"
#include "Builder.h"
#include "Type.h"
#include "Wood.h"

using namespace std;

void Inventory::add_instrument(string serial_number, double price,
                               InstrumentSpec *spec)
{
    Instrument *instrument = nullptr;

    if (typeid(spec) == typeid(GuitarSpec))
    {
        instrument = new Guitar(serial_number, price,
                                dynamic_cast<GuitarSpec *>(spec));
    }
    else if (typeid(spec) == typeid(MandolinSpec))
    {
        instrument = new Mandolin(serial_number, price,
                                  dynamic_cast<MandolinSpec *>(spec));
    }

    inventory.push_back(instrument);
}

Instrument *Inventory::get(string serial_number)
{
    for (Instrument *instrument : inventory)
    {
        if (instrument->get_serial_number() == serial_number) return instrument;
    }

    return nullptr;
}

vector<Guitar *> Inventory::search(GuitarSpec *search_spec)
{
    vector<Guitar *> matching_guitars;

    for (Instrument *instrument : inventory)
    {
        Guitar *guitar = dynamic_cast<Guitar *>(instrument);

        if (guitar != nullptr)
        {
            GuitarSpec *guitar_spec =
                            dynamic_cast<GuitarSpec *>(guitar->get_spec());

            if (guitar_spec->matches(search_spec))
            {
                matching_guitars.push_back(guitar);
            }
        }
    }

    return matching_guitars;
}

vector<Mandolin *> Inventory::search(MandolinSpec *search_spec)
{
    vector<Mandolin *> matching_mandolins;

    for (Instrument *instrument : inventory)
    {
        Mandolin *mandolin = dynamic_cast<Mandolin *>(instrument);

        if (mandolin != nullptr)
        {
            MandolinSpec *mandolin_spec =
                            dynamic_cast<MandolinSpec *>(mandolin->get_spec());

            if (mandolin_spec->matches(search_spec))
            {
                matching_mandolins.push_back(mandolin);
            }
        }
    }

    return matching_mandolins;
}
